# Generated by Django 3.1.1 on 2021-01-04 15:25

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('moons', '0004_auto_20201205_0626'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='miningobservation',
            name='frack',
        ),
    ]



# Generated by Django 3.2.5 on 2021-09-06 09:15

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('moons', '0014_invoicerecord_tax_dump'),
    ]

    operations = [
        migrations.AddField(
            model_name='invoicerecord',
            name='total_mined',
            field=models.DecimalField(decimal_places=2, default=0, max_digits=20),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='invoicerecord',
            name='total_taxed',
            field=models.DecimalField(decimal_places=2, default=0, max_digits=20),
            preserve_default=False,
        ),
    ]



from datetime import datetime, timedelta
from django.core.exceptions import ObjectDoesNotExist
from django.contrib.auth.models import User
from django.db import models
from allianceauth.authentication.models import CharacterOwnership
from allianceauth.eveonline.models import EveCharacter, EveCorporationInfo
from corptools.models import CorporationAudit, EveItemType, MapConstellation, MapRegion, MapSystem, MapSystemMoon, EveLocation, Notification, EveName
from django.db.models import Subquery, OuterRef
from django.db.models import FloatField, F, ExpressionWrapper

from . import app_settings
from django.utils import timezone

from invoices.models import Invoice
from .managers import MoonManager

if app_settings.discord_bot_active():
    import aadiscordbot

from django.forms import model_to_dict
from django.core.serializers.json import DjangoJSONEncoder
from django.db.models import Model

import logging
import copy
import json

logger = logging.getLogger(__name__)


class MoonFrack(models.Model):
    objects = MoonManager()

    corporation = models.ForeignKey(CorporationAudit, on_delete=models.CASCADE, related_name='moon')

    moon_name = models.ForeignKey(MapSystemMoon, on_delete=models.SET_NULL, null=True, default=None)
    moon_id = models.IntegerField()

    notification = models.ForeignKey(Notification, on_delete=models.SET_NULL, null=True, default=None)

    structure = models.ForeignKey(EveLocation, on_delete=models.CASCADE)

    start_time = models.DateTimeField()
    arrival_time = models.DateTimeField()
    auto_time = models.DateTimeField()

    #frack_pinged = models.BooleanField(default=False)
    #finish_notification = models.ForeignKey(Notification, on_delete=models.SET_NULL, null=True, default=None)
    #frack_notification = models.ForeignKey(Notification, on_delete=models.SET_NULL, null=True, default=None)
    
    class Meta:
        unique_together = (('arrival_time', 'moon_id'),)

    def __str__(self):
        return "{} - {}".format(self.moon_name.name, self.arrival_time)

    class Meta:
        permissions = (('view_available', 'Can View Configured Public Moons'),
                       ('view_corp', 'Can View Own Corps Moons'),  # these do nothing as of now.
                       ('view_alliance', 'Can View Own Alliances Moons'),
                       ('view_all', 'Can View All Moons'))


class FrackOre(models.Model):
    frack = models.ForeignKey(MoonFrack, on_delete=models.CASCADE, related_name='frack')
    ore = models.ForeignKey(EveItemType, on_delete=models.CASCADE)
    total_m3 = models.DecimalField(max_digits=20, decimal_places=2)


# Corp Mining Observation
class MiningObservation(models.Model):
    ob_pk = models.CharField(max_length=50, primary_key=True)

    observing_corporation = models.ForeignKey(CorporationAudit, on_delete=models.SET_NULL, null=True, default=None)

    observing_id = models.BigIntegerField(null=True, default=None, blank=True)
    structure = models.ForeignKey(EveLocation, on_delete=models.CASCADE, null=True, default=None, blank=True)
    moon = models.ForeignKey(MapSystemMoon, on_delete=models.SET_NULL, null=True, default=None, blank=True)

    character_name = models.ForeignKey(EveName, on_delete=models.SET_NULL, null=True, default=None)

    character_id = models.IntegerField()
    last_updated = models.DateTimeField()
    quantity = models.BigIntegerField()
    recorded_corporation_id = models.IntegerField()

    type_name = models.ForeignKey(EveItemType, on_delete=models.SET_NULL, null=True, default=None) 
    type_id = models.IntegerField()

    @classmethod
    def build_pk(cls, corp_id, observer_id, observed_character_id, ob_date, type_id):
        """
        Helper method to get a unique pk for a specific observation. Usefull for bulk updates of data.
        :param corp_id: Observer Corp.
        :param observer_id: Mining Observer ID.
        :param observed_character_id: Character Observed.
        :param ob_date: DateTime of observation.
        :return: :class:String
        """
        date_str = ob_date.strftime("%Y%m%d")
        return "{}-{}-{}-{}-{}".format(corp_id,
                                    observer_id,
                                    observed_character_id,
                                    date_str,
                                    type_id)

    class Meta:
        indexes = (
            models.Index(fields=['observing_id']),
            models.Index(fields=['last_updated']),
            models.Index(fields=['recorded_corporation_id']),
            models.Index(fields=['type_id']),
            models.Index(fields=['character_id'])
        )


    @classmethod
    def tax_moons(cls, start, end):
        #get all tax items and return the tax for the time period.

        player_data = {}
        observerd_ids = []
        observers_taxed = []

        taxes = MiningTax.objects.all().order_by('-rank')

        for tax in taxes:

            type_price = OrePrice.objects.filter(item_id=OuterRef('type_id'))

            observed = MiningObservation.objects.select_related('structure', 'type_name', 'structure__system', 'structure__system__constellation', 'character_name').all() \
                .annotate(isk_value=ExpressionWrapper(
                    Subquery(type_price.values('price')) * F('quantity'),
                        output_field=FloatField())) 
            
            if tax.use_variable_tax:
                tax_price = OreTax.objects.filter(item_id=OuterRef('type_id'), tax=tax.tax_rate)
                observed = observed.annotate(tax_value=ExpressionWrapper(
                    Subquery(tax_price.values('price')) * F('quantity'),
                        output_field=FloatField())) 
                
            observed = observed.filter(last_updated__gte=start).filter(last_updated__lt=end)

            if tax.corp:
                observed = observed.filter(observing_corporation__corporation=tax.corp)
            if tax.region:
                observed = observed.filter(structure__system__constellation__region=tax.region)
            if tax.constellation:
                observed = observed.filter(structure__system__constellation=tax.constellation)
            if tax.system:
                observed = observed.filter(structure__system=tax.system)
            if tax.moon:
                observed = observed.filter(moon=tax.moon)

            rate = float(tax.flat_tax_rate)
            # do the ranks
            observed = observed.exclude(structure__in=observers_taxed)
            #print(observed.query)
            #print(observed.count())

            for i in observed.distinct():
                if i.structure not in observers_taxed:
                    observers_taxed.append(i.structure)
                if i.ob_pk not in observerd_ids:
                    observerd_ids.append(i.ob_pk)
                    if i.character_name.eve_id not in player_data:
                        player_data[i.character_name.eve_id] = {}
                        player_data[i.character_name.eve_id]['ores'] = {}
                        player_data[i.character_name.eve_id]['totals_isk'] = 0
                        player_data[i.character_name.eve_id]['tax_isk'] = 0
                        player_data[i.character_name.eve_id]['character_model'] = i.character_name
                        player_data[i.character_name.eve_id]["seen_at"] = []
                    
                    if i.structure.location_name not in player_data[i.character_name.eve_id]["seen_at"]:
                        player_data[i.character_name.eve_id]["seen_at"].append(i.structure.location_name)

                    player_data[i.character_name.eve_id]['totals_isk'] =player_data[i.character_name.eve_id]['totals_isk'] + i.isk_value
                    
                    if tax.use_variable_tax:
                        player_data[i.character_name.eve_id]['tax_isk'] = player_data[i.character_name.eve_id]['tax_isk'] + i.tax_value
                    else:
                        player_data[i.character_name.eve_id]['tax_isk'] = player_data[i.character_name.eve_id]['tax_isk'] + i.isk_value * rate
                            

                    if i.type_name not in player_data[i.character_name.eve_id]['ores']: 
                        player_data[i.character_name.eve_id]['ores'][i.type_name.name] = {}
                        player_data[i.character_name.eve_id]['ores'][i.type_name.name]["type_id"] = i.type_id
                        player_data[i.character_name.eve_id]['ores'][i.type_name.name]["value"] = i.isk_value
                        player_data[i.character_name.eve_id]['ores'][i.type_name.name]["count"] = i.quantity
                    else:
                        player_data[i.character_name.eve_id]['ores'][i.type_name.name]["value"] = player_data[i.character_name.eve_id]['ores'][i.type_name.name]["value"]+i.isk_value
                        player_data[i.character_name.eve_id]['ores'][i.type_name.name]["count"] = player_data[i.character_name.eve_id]['ores'][i.type_name.name]["count"]+i.quantity


        output = {
            'player_data': player_data
        }

        return output


class OreTaxRates(models.Model):
    tag = models.CharField(max_length=500, default="Mining Tax")
    refine_rate = models.DecimalField(
        max_digits=5, decimal_places=2, default=87.5)
    ore_rate = models.DecimalField(max_digits=5, decimal_places=2)  # normal
    ubiquitous_rate = models.DecimalField(
        max_digits=5, decimal_places=2)  # ubiq
    common_rate = models.DecimalField(max_digits=5, decimal_places=2)  # comon
    uncommon_rate = models.DecimalField(
        max_digits=5, decimal_places=2)  # uncom
    rare_rate = models.DecimalField(max_digits=5, decimal_places=2)  # rare
    exceptional_rate = models.DecimalField(
        max_digits=5, decimal_places=2)  # best

    def __str__(self):
        try:
            return self.tag
        except:
            return "Mining Tax"


# Market History ( GMetrics )
class OrePrice(models.Model):
    item = models.ForeignKey(EveItemType, on_delete=models.DO_NOTHING, related_name='ore_price')
    price = models.DecimalField(max_digits=20, decimal_places=2)
    last_update = models.DateTimeField(auto_now=True)


# tax rates History
class OreTax(models.Model):
    item = models.ForeignKey(EveItemType, on_delete=models.DO_NOTHING, related_name='ore_tax')
    price = models.DecimalField(max_digits=20, decimal_places=2)
    last_update = models.DateTimeField(auto_now=True)
    tax = models.ForeignKey(OreTaxRates, on_delete=models.CASCADE, related_name='tax_rate')


class MiningTax(models.Model):
    corp = models.ForeignKey(
        EveCorporationInfo, on_delete=models.CASCADE, related_name='moon_mining_tax', null=True, default=None, blank=True)
    tax_rate = models.ForeignKey(
        OreTaxRates, on_delete=models.CASCADE, null=True, default=None, blank=True)
    use_variable_tax = models.BooleanField(default=False)
    flat_tax_rate = models.DecimalField(
        max_digits=5, decimal_places=2, default=0.0)  # best
    region = models.ForeignKey(
        MapRegion, on_delete=models.CASCADE, related_name='tax_region', null=True, default=None, blank=True)
    constellation = models.ForeignKey(
        MapConstellation, on_delete=models.CASCADE, related_name='tax_constellation', null=True, default=None, blank=True)
    system = models.ForeignKey(
        MapSystem, on_delete=models.CASCADE, related_name='tax_system', null=True, default=None, blank=True)
    moon = models.ForeignKey(
        MapSystemMoon, on_delete=models.CASCADE, related_name='tax_moon', null=True, default=None, blank=True)
    rank = models.IntegerField(default=0, null=True, blank=True)

    def __str__(self):
        area = "Everywhere"
        if self.region:
            area = f"Region: {self.region.name}"
        elif self.constellation:
            area = f"Constellation: {self.constellation.name}"
        elif self.system:
            area = f"System: {self.system.name}"
        elif self.moon:
            area = f"Moon': {self.moon.name}"
        
        corp = "Everyone"
        if self.corp:
            corp = self.corp.corporation_name

        # return
        rate = ""
        if self.use_variable_tax:
            rate = " Variable ({})".format(self.tax_rate.tag)
        else:
            rate = "{}%".format(self.flat_tax_rate*100)
        return "#{3}: Mining Tax {0} for {1}, {2}".format(rate, corp, area, self.rank)


# tax rates History
class InvoiceRecord(models.Model):
    start_date = models.DateTimeField()
    end_date = models.DateTimeField()
    ore_prices = models.TextField()
    tax_dump = models.TextField()
    total_mined = models.DecimalField(max_digits=20, decimal_places=2)
    total_taxed = models.DecimalField(max_digits=20, decimal_places=2)
    base_ref = models.CharField(max_length=72, default="")

    @classmethod
    def sanitize_date(cls, date):
        return datetime(year=date.year, 
                        month=date.month, 
                        day=date.day, 
                        tzinfo=date.tzinfo, 
                        hour=0,
                        minute=0,
                        second=0)


    @classmethod
    def get_last_invoice_date(cls):
        try:
            return InvoiceRecord.objects.all().order_by('-end_date').first().end_date
        except (ObjectDoesNotExist, AttributeError) as e:
            return datetime.min
    

    @classmethod
    def generate_invoice(cls, character, ref, amount, message):
        #generate an invoice and return it
        due = timezone.now() + timedelta(days=14)
        return Invoice(character=character,
                       amount=amount,
                       invoice_ref=ref,
                       note=message,
                       due_date=due)


    @classmethod
    def ping_invoice(cls, inv):
        # ping the invoice to the user ( if we know them )
        message = "\nPlease check auth for how to pay your mining Taxes!"
        inv.notify(message, title="Mining Taxes")


    @classmethod
    def generate_message(cls, raw_data):
        # Mining data for the invoice message, characters and stations
        pass


    @classmethod
    def generate_inv_ref(cls, char_id, start, end):
        date_str = "%Y%m%d"
        start_str = start.strftime(date_str)
        end_str = end.strftime(date_str)
        return f"MT{char_id}-{start_str}-{end_str}"


    @classmethod
    def generate_invoice_data(cls):
        start_date = cls.sanitize_date(cls.get_last_invoice_date())
        end_date = cls.sanitize_date(timezone.now())
        taxes = {}

        tax_data = MiningObservation.tax_moons(start_date, end_date)
        p_d = copy.deepcopy(tax_data['player_data'])

        all_ownerships = CharacterOwnership.objects.filter(character__character_id__in=tax_data['player_data'].keys()) 

        for o in all_ownerships:
            if o.user.id not in taxes:
                taxes[o.user.id] = {
                    "user": o.user,
                    "locations": set(),
                    "characters": set(),
                    "total_value": 0,
                    "tax_value": 0,
                    "ref": cls.generate_inv_ref(o.user.id, start_date, end_date)
                }

            tx = p_d.pop(o.character.character_id)

            taxes[o.user.id]['characters'].add(tx.get('character_model').name)
            taxes[o.user.id]['locations'].update(tx.get('seen_at'))
            taxes[o.user.id]['total_value'] += tx.get('totals_isk')
            taxes[o.user.id]['tax_value'] += tx.get('tax_isk')
            
            del tx

        return {"knowns": taxes, 
                "unknowns": p_d, 
                "start": start_date, 
                "end": end_date}


    @classmethod
    def generate_invoices(cls):
        from moons.helpers import OreHelper
        data = cls.generate_invoice_data()
        total_mined = 0
        total_taxed = 0
        # run known people
        for u, d in data['knowns'].items():
            try:
                ref = d['ref']
                amount = d['tax_value']

                total_mined += d['total_value']
                total_taxed += d['tax_value']

                character = d['user'].profile.main_character

                message = f"Mining Taxes for: {', '.join(d['characters'])}\nAt: {', '.join(d['locations'])}"
                inv = cls.generate_invoice(character, ref, amount, message)
                inv.save()
                cls.ping_invoice(inv)
            except KeyError:
                pass # probably wanna ping admin about it.
        
        for u, d in data['unknowns'].items():
            try:
                ref = cls.generate_inv_ref(d['character_model'].eve_id, data['start'], data['end'])
                amount = d['tax_isk']

                total_mined += d['totals_isk']
                total_taxed += d['tax_isk']

                try: 
                    character = EveCharacter.objects.get(character_id=d['character_model'].eve_id)
                except EveCharacter.DoesNotExist:
                    character = EveCharacter.objects.create_character(character_id=d['character_model'].eve_id)

                message = f"Mining Taxes for: {character.character_name} \nAt: {', '.join(d['seen_at'])}"
                inv = cls.generate_invoice(character, ref, amount, message)
                inv.save()
            except KeyError:
                pass # probably wanna ping admin about it.

        return cls.objects.create(start_date= data['start'],
                           end_date= data['end'],
                           tax_dump= json.dumps(data, cls=ExtendedJsonEncoder),
                           ore_prices= json.dumps(OreHelper.get_ore_array_with_value(), cls=ExtendedJsonEncoder),
                           total_mined=total_mined,
                           total_taxed=total_taxed,
                           base_ref=cls.generate_inv_ref("[id]",  data['start'],  data['end'])
                           )


class ExtendedJsonEncoder(DjangoJSONEncoder):

    def default(self, o):

        if isinstance(o, User):
            return {"user_id": o.pk}

        if isinstance(o, Model):
            return model_to_dict(o)

        if isinstance(o, set):
            return list(o)

        return super().default(o)



from .models import OrePrice, OreTax, MiningTax
from corptools.models import EveItemType, InvTypeMaterials
from django.db.models import Subquery, OuterRef


class OreHelper:

    rank_ids = {
        1923: "exceptional_rate",
        1922: "rare_rate",
        1921: "uncommon_rate",
        1920: "common_rate",
        1884: "ubiquitous_rate"
    }

    asteroids = 25

    def get_mineral_array():   
        return list(set(InvTypeMaterials.objects.filter(eve_type__group__category_id=OreHelper.asteroids) \
            .values_list('met_type_id', flat=True)))

    def get_ore_array():     
        inv_types = InvTypeMaterials.objects.filter(eve_type__group__category_id=OreHelper.asteroids) \
            .select_related('eve_type', 'eve_type__group', 'met_type')
        
        ore_infos = {}
        for comp in inv_types:
            if comp.eve_type.name not in ore_infos:
                rarity = OreHelper.rank_ids[comp.eve_type.group_id] if comp.eve_type.group_id in OreHelper.rank_ids else "ore_rate"
                ore_infos[comp.eve_type.pk] = {
                    "minerals": {},
                    "volume": comp.eve_type.packaged_volume,
                    "model": comp.eve_type,
                    "rarity": rarity,
                    "portion": comp.eve_type.portion_size
                }
            ore_infos[comp.eve_type.pk]['minerals'][comp.met_type.name] = comp.qty
        
        return ore_infos


    def get_ore_array_with_value():
        input = OreHelper.get_ore_array()
        for o in OrePrice.objects.all():
            input[o.item_id]['value'] = o.price

        return input


    def get_ore_array_with_value_and_taxs():
        input = OreHelper.get_ore_array_with_value()
        for t in MiningTax.objects.all():
            input[o.item_id]['value'] = o.price

        return input



# Generated by Django 3.2.5 on 2021-08-30 11:21

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('moons', '0011_oretax_tax'),
    ]

    operations = [
        migrations.RenameField(
            model_name='oretaxrates',
            old_name='excptional_rate',
            new_name='exceptional_rate',
        ),
    ]



import logging
import json
from datetime import timedelta, datetime
import yaml
import requests 

from celery import shared_task, chain
from django.utils import timezone
from . import app_settings
from .helpers import OreHelper
from .models import MiningObservation, MoonFrack, FrackOre, OrePrice, OreTaxRates, OreTax, InvoiceRecord
from corptools.models import Notification, EveLocation, MapSystemMoon, EveItemType, CorporationAudit, EveName
from corptools import providers
from corptools.task_helpers.corp_helpers import get_corp_token
from corptools.task_helpers.update_tasks import fetch_location_name
from allianceauth.services.tasks import QueueOnce
from .app_settings import PUBLIC_MOON_CORPS

logger = logging.getLogger(__name__)

def filetime_to_dt(ft):
    us = (ft - 116444736000000000) // 10
    return datetime(1970, 1, 1) + timedelta(microseconds=us)

@shared_task
def process_moon_pulls():
    logger.debug("Started Mining Pull Sync")
    start_time = timezone.now() - timedelta(days=65)

    notification_type_filter = ['MoonminingExtractionStarted',]

    # YAML Structure
    #    autoTime: #####
    #    moonID: ####
    #    oreVolumeByType:
    #        45490: 3981232.1291307067
    #        45499: 7115599.461194293
    #        45504: 4589835.07632598
    #    readyTime: #####
    #    solarSystemID: ####
    #    startedBy: ####
    #    startedByLink: <a href="showinfo:1385//###">CCCC</a>
    #    structureID: ####
    #    structureLink: <a href="showinfo:35835//####">CCCC</a>
    #    structureName: CCCCCCCCC
    #    structureTypeID: ####

    notifications = Notification.objects.filter(character__character__corporation_id__in=PUBLIC_MOON_CORPS,
                                                timestamp__gte=start_time,
                                                notification_type__in=notification_type_filter)

    fracks = MoonFrack.objects.filter(start_time__gte=start_time)
    notification_ids = set(fracks.values_list('notification__notification_id', flat=True))
    frack_dict = {}
    for frack in fracks:
        if frack.moon_id not in frack_dict:
            frack_dict[frack.moon_id] = {}
        frack_dict[frack.moon_id][frack.start_time] = frack

    _type_list = []
    _ores = []
    for notification in notifications:
        try:
            if notification.notification_id not in notification_ids:
                notification_ids.add(notification.notification_id)
                notification_data = yaml.load(notification.notification_text)
                moon_id = notification_data['moonID']
                start_time = notification.timestamp
            
                new_frack = False

                if moon_id in frack_dict:
                    if start_time not in frack_dict[moon_id]:
                        new_frack = True
                else:
                    new_frack = True
                
                if new_frack:
                    _structure, _c = EveLocation.objects.update_or_create(location_id=notification_data['structureID'],
                                                                        defaults={
                                                                            "location_name":notification_data['structureName'],
                                                                            "system_id":notification_data['solarSystemID']
                                                                        })
                    _moon, _c = MapSystemMoon.objects.get_or_create_from_esi(moon_id=moon_id)
                    _corp_audit = CorporationAudit.objects.get(corporation__corporation_id=notification.character.character.corporation_id)
                    _frack = MoonFrack.objects.create(corporation=_corp_audit,
                                                    moon_name=_moon,
                                                    moon_id=moon_id,
                                                    notification=notification,
                                                    structure=_structure,
                                                    start_time=start_time,
                                                    arrival_time=filetime_to_dt(notification_data['readyTime']),
                                                    auto_time=filetime_to_dt(notification_data['autoTime'])
                                                    )

                    for ore, volume in notification_data['oreVolumeByType'].items():
                        if ore not in _type_list:
                            _type_list.append(ore)
                        _ores.append(FrackOre(frack=_frack,
                                            ore_id=ore,
                                            total_m3=volume
                                            ))
        except Exception as e:
            logger.warn("Failed to process moon", exc_info=1)
            pass

    EveItemType.objects.create_bulk_from_esi(_type_list)
    FrackOre.objects.bulk_create(_ores, batch_size=500)
    return "fetched!"


@shared_task
def run_obs_for_all_corps(force=False):
    for c in CorporationAudit.objects.all():
        logger.debug(f"Sending Obs update for {c.corporation}")
        queue_moon_obs.apply_async(args=[c.corporation.corporation_id], priority=6)


@shared_task
def queue_moon_obs(corp_id, force=False):
    logger.debug("Started Mining Ob Sync for {}".format(corp_id))

    token = get_corp_token(corp_id, ['esi-industry.read_corporation_mining.v1'], ['Accountant', 'Director'])
    obs = providers.esi.client.Industry.get_corporation_corporation_id_mining_observers(corporation_id=corp_id,
                                                                                             token=token.valid_access_token()).results()
    task_queue = []
    for ob in obs:
        task_queue.append(process_moon_obs.si(ob.get('observer_id'), corp_id))
    chain(task_queue).apply_async(priority=8)


@shared_task
def process_moon_obs(observer_id, corporation_id):
    logger.debug("Started Mining Ob Sync for {}".format(observer_id))

    token = get_corp_token(corporation_id, ['esi-industry.read_corporation_mining.v1', 'esi-universe.read_structures.v1'], ['Accountant', 'Director'])
    obs = providers.esi.client.Industry.get_corporation_corporation_id_mining_observers_observer_id(corporation_id=corporation_id,
                                                                                                   observer_id=observer_id,
                                                                                                   token=token.valid_access_token()).results()
    eve_names = set(EveName.objects.all().values_list('eve_id', flat=True))
    type_names = set(EveItemType.objects.all().values_list('type_id', flat=True))
    corp = CorporationAudit.objects.get(corporation__corporation_id=corporation_id)

    observer = None
    structure_exists = False
    moon = None

    try:
        observer = EveLocation.objects.get(location_id=observer_id)
        structure_exists = True

    except EveLocation.DoesNotExist:
        _ob = fetch_location_name(observer_id, "solar_system", token.character_id)
        if _ob:
            _ob.save()
            observer = _ob
            structure_exists = True

    _moon = MoonFrack.objects.filter(structure_id=observer_id, moon_name__isnull=False)
    if _moon.exists():
        moon = _moon.last().moon_name

    ob_pks = set(MiningObservation.objects.filter(observing_id=observer_id).values_list('ob_pk', flat=True))
    mining_ob_updates = []
    mining_ob_creates = []
    new_eve_names = []
    new_item_types = []
    for ob in obs:
        pk = MiningObservation.build_pk(corporation_id, observer_id, ob.get('character_id'), ob.get('last_updated'), ob.get('type_id'))
        
        if ob.get('character_id') not in eve_names:
            new_eve_names.append(ob.get('character_id'))
            eve_names.add(ob.get('character_id'))

        if ob.get('type_id') not in type_names:
            new_item_types.append(ob.get('type_id'))
            type_names.add(ob.get('type_id'))

        _ob = MiningObservation(ob_pk=pk,
                                observing_id=observer_id,
                                observing_corporation=corp,
                                character_name_id=ob.get('character_id'),
                                character_id=ob.get('character_id'),
                                last_updated=ob.get('last_updated'),
                                quantity=ob.get('quantity'),
                                recorded_corporation_id=ob.get('recorded_corporation_id'),
                                type_id=ob.get('type_id'),
                                type_name_id=ob.get('type_id'),
                                moon=moon)

        if structure_exists:
            _ob.structure = observer

        if pk in ob_pks:
            mining_ob_updates.append(_ob)
        else: 
            mining_ob_creates.append(_ob)

    EveName.objects.create_bulk_from_esi(new_eve_names)
    EveItemType.objects.create_bulk_from_esi(new_item_types)

    if len(mining_ob_creates) > 0:
        MiningObservation.objects.bulk_create(mining_ob_creates)
    
    if len(mining_ob_updates) > 0:
        MiningObservation.objects.bulk_update(mining_ob_updates, ['quantity', 'last_updated', 'structure', 'observing_corporation'])

    msg = f"Corp:{corporation_id} Moon:{observer_id} Updated:{len(mining_ob_updates)} Created:{len(mining_ob_creates)}"
    logger.debug(msg)
    return msg


@shared_task
def update_ore_prices():
    mins = OreHelper.get_mineral_array()
    minstr = str(mins.pop())
    for i in mins:
        minstr += f",{i}"
    url = f"https://market.fuzzwork.co.uk/aggregates/?station=60003760&types={minstr}"
    response = requests.get(url)
    price_data = response.json()
    price_cache = {}
    for key, item in price_data.items():
        name, created = EveItemType.objects.get_or_create_from_esi(key)
        name = name.name
        if name not in price_cache:
            price_cache[name] = {}
        price_cache[name]['the_forge'] = float(item['buy']['weightedAverage'])

    ores = OreHelper.get_ore_array()

    price_source = 'the_forge'  # We only use this now.

    for ore, minerals in ores.items():
        price = 0
        for mineral, qty in minerals["minerals"].items():
            price = price + (qty * price_cache[mineral][price_source])

        OrePrice.objects.update_or_create(item=minerals['model'],
                                          defaults={
                                            "price": price/minerals['portion']
                                          })

    update_tax_prices.apply_async(priority=7)

    return json.dumps(price_cache)


@shared_task
def update_tax_prices():
    taxs = OreTaxRates.objects.all()

    ores = OreHelper.get_ore_array_with_value()

    for tax in taxs:
        for id, o in ores.items():
            rate = getattr(tax, o['rarity'])
            price = float(o['value']) * (float(rate)/100) * (float(tax.refine_rate)/100)
            OreTax.objects.update_or_create(
                item=o['model'],
                tax=tax,
                defaults={
                    "price": price
                }
            )


@shared_task
def generate_taxes():
    taxes = InvoiceRecord.generate_invoices()
    return f"Taxes Generated {taxes.base_ref} Total Mined:{taxes.total_mined:,} Total Tax:{taxes.total_taxed:,}"



from allianceauth.services.hooks import MenuItemHook, UrlHook
from django.utils.translation import ugettext_lazy as _
from allianceauth import hooks
from . import urls

class MoonsHook(MenuItemHook):
    def __init__(self):
        MenuItemHook.__init__(self,
                              _('Moon Board'),
                              'fas fa-moon fa-fw',
                              'moons:list',
                              navactive=['moons:'])

    def render(self, request):
        if request.user.has_perm('moons.access_moons'):
            return MenuItemHook.render(self, request)
        return ''

@hooks.register('menu_item_hook')
def register_menu():
    return MoonsHook()

@hooks.register('url_hook')
def register_url():
    return UrlHook(urls, 'moons', r'^moons/')



from unittest import mock
from django.test import TestCase
from allianceauth.tests.auth_utils import AuthUtils
from ..models import Invoice
from django.contrib.auth.models import Permission
from django.utils import timezone
from allianceauth.eveonline.models import EveCharacter
from allianceauth.authentication.models import CharacterOwnership

class TestInvoicesAccessPerms(TestCase): 
    def setUp(cls):
        cls.user1 = AuthUtils.create_user('Main User1')
        cls.user2 = AuthUtils.create_user('Main User2')
        cls.user3 = AuthUtils.create_user('Main User3')
        cls.user4 = AuthUtils.create_user('Main User4')
        cls.user5 = AuthUtils.create_user('Main User5')
        
        char1 = EveCharacter.objects.create(character_id=1,
            character_name='character.name1',
            corporation_id=123,
            corporation_name='corporation.name1',
            corporation_ticker='ABC')
        cls.ci1 = Invoice.objects.create(character=char1, amount=5, due_date=timezone.now())

        cls.user1.profile.main_character = char1
        CharacterOwnership.objects.create(user=cls.user1, character=char1, owner_hash="abc123")
        cls.user1.profile.save()

        char2 = EveCharacter.objects.create(character_id=2,
            character_name='character.name2',
            corporation_id=123,
            corporation_name='corporation.name1',            
            corporation_ticker='ABC')
        cls.ci2 = Invoice.objects.create(character=char2, amount=5, due_date=timezone.now())
        CharacterOwnership.objects.create(user=cls.user1, character=char2, owner_hash="cba123")
        cls.user1.profile.refresh_from_db()

        char3 = EveCharacter.objects.create(character_id=3,
            character_name='character.name3',
            corporation_id=2,
            corporation_name='corporation.name2',            
            corporation_ticker='ABC2',            
            alliance_id=3, 
            alliance_name='test alliance2', 
            alliance_ticker='TEST')
        cls.ci3 = Invoice.objects.create(character=char3, amount=5, due_date=timezone.now())

        cls.user2.profile.main_character = char3
        CharacterOwnership.objects.create(user=cls.user2, character=char3, owner_hash="cba321")

        cls.user2.profile.save()
        cls.user2.profile.refresh_from_db()
        
        char4 = EveCharacter.objects.create(character_id=4,
            character_name='character.name4',
            corporation_id=2,
            corporation_name='corporation.name2',            
            corporation_ticker='ABC',            
            alliance_id=3, 
            alliance_name='test alliance2', 
            alliance_ticker='TEST')
        cls.ci4 = Invoice.objects.create(character=char4, amount=5, due_date=timezone.now())

        char5 = EveCharacter.objects.create(character_id=5,
            character_name='character.name5',
            corporation_id=3,
            corporation_name='corporation.name3',            
            corporation_ticker='ABC3',            
            alliance_id=4, 
            alliance_name='test alliance2', 
            alliance_ticker='TEST')
        cls.ci5 = Invoice.objects.create(character=char5, amount=5, due_date=timezone.now())

        cls.user3.profile.main_character = char5
        CharacterOwnership.objects.create(user=cls.user3, character=char5, owner_hash="abc432")

        cls.user3.profile.save()
        cls.user3.profile.refresh_from_db()

        char6 = EveCharacter.objects.create(character_id=6,
            character_name='character.name6',
            corporation_id=3,
            corporation_name='corporation.name3',            
            corporation_ticker='ABC3',            
            alliance_id=4, 
            alliance_name='test alliance2', 
            alliance_ticker='TEST2')
        cls.ci6 = Invoice.objects.create(character=char6, amount=5, due_date=timezone.now())

        char7 = EveCharacter.objects.create(character_id=7,
            character_name='character.name7',
            corporation_id=4,
            corporation_name='corporation.name4',            
            corporation_ticker='ABC4',            
            alliance_id=4, 
            alliance_name='test alliance2', 
            alliance_ticker='TEST2')
        cls.ci7 = Invoice.objects.create(character=char7, amount=5, due_date=timezone.now())
        CharacterOwnership.objects.create(user=cls.user3, character=char7, owner_hash="def432")

        char8 = EveCharacter.objects.create(character_id=8,
            character_name='character.name8',
            corporation_id=5,
            corporation_name='corporation.name5',            
            corporation_ticker='ABC5',            
            alliance_id=3, 
            alliance_name='test alliance3', 
            alliance_ticker='TEST3')
        cls.ci8 = Invoice.objects.create(character=char8, amount=5, due_date=timezone.now())

        cls.view_corp_permission = Permission.objects.get_by_natural_key('view_corp', 'invoices', 'invoice')
        cls.view_alliance_permission = Permission.objects.get_by_natural_key('view_alliance', 'invoices', 'invoice')
        cls.view_all_permission = Permission.objects.get_by_natural_key('view_all', 'invoices', 'invoice')

        char9 = EveCharacter.objects.create(character_id=9,
            character_name='character.name9',
            corporation_id=4,
            corporation_name='corporation.name4',            
            corporation_ticker='ABC4')
        cls.ci9 = Invoice.objects.create(character=char9, amount=5, due_date=timezone.now())
        cls.user4.profile.main_character = char9
        CharacterOwnership.objects.create(user=cls.user4, character=char9, owner_hash="tre456")
        cls.user4.profile.save()


    def test_no_perms_get_self_u1(self):  # always get self.
        cs = Invoice.objects.visible_to(self.user1)
        self.assertIn(self.ci1, cs)
        self.assertIn(self.ci2, cs)
        self.assertNotIn(self.ci3, cs)
        self.assertNotIn(self.ci4, cs)
        self.assertNotIn(self.ci5, cs)
        self.assertNotIn(self.ci6, cs)
        self.assertNotIn(self.ci7, cs)
        self.assertNotIn(self.ci8, cs)

    def test_no_perms_get_self_u2(self):  # always get self.
        cs = Invoice.objects.visible_to(self.user2)
        self.assertNotIn(self.ci1, cs)
        self.assertNotIn(self.ci2, cs)
        self.assertIn(self.ci3, cs)
        self.assertNotIn(self.ci4, cs)
        self.assertNotIn(self.ci5, cs)
        self.assertNotIn(self.ci6, cs)
        self.assertNotIn(self.ci7, cs)
        self.assertNotIn(self.ci8, cs)

    def test_no_perms_get_self_u3(self):  # always get self.
        cs = Invoice.objects.visible_to(self.user3)
        self.assertNotIn(self.ci1, cs)
        self.assertNotIn(self.ci2, cs)
        self.assertNotIn(self.ci3, cs)
        self.assertNotIn(self.ci4, cs)
        self.assertIn(self.ci5, cs)
        self.assertNotIn(self.ci6, cs)
        self.assertIn(self.ci7, cs)
        self.assertNotIn(self.ci8, cs)

    def test_no_perms_get_self_in_alliance(self):
        cs = Invoice.objects.visible_to(self.user2)
        self.assertNotIn(self.ci1, cs)
        self.assertNotIn(self.ci2, cs)
        self.assertIn(self.ci3, cs)
        self.assertNotIn(self.ci4, cs)
        self.assertNotIn(self.ci5, cs)
        self.assertNotIn(self.ci6, cs)
        self.assertNotIn(self.ci7, cs)
        self.assertNotIn(self.ci8, cs)

    def test_get_corp_in_alliance(self):
        self.user2.user_permissions.add(self.view_corp_permission)
        cs = Invoice.objects.visible_to(self.user2)
        self.assertNotIn(self.ci1, cs)
        self.assertNotIn(self.ci2, cs)
        self.assertIn(self.ci3, cs)
        self.assertIn(self.ci4, cs)
        self.assertNotIn(self.ci5, cs)
        self.assertNotIn(self.ci6, cs)
        self.assertNotIn(self.ci7, cs)
        self.assertNotIn(self.ci8, cs)

    def test_get_alliance_no_alliance(self):
        self.user4.user_permissions.add(self.view_alliance_permission)
        cs = Invoice.objects.visible_to(self.user4)
        self.assertNotIn(self.ci1, cs)
        self.assertNotIn(self.ci2, cs)
        self.assertNotIn(self.ci3, cs)
        self.assertNotIn(self.ci4, cs)
        self.assertNotIn(self.ci5, cs)
        self.assertNotIn(self.ci6, cs)
        self.assertIn(self.ci7, cs)
        self.assertNotIn(self.ci8, cs)
        self.assertIn(self.ci9, cs)

    def test_get_alliance(self):
        self.user3.user_permissions.add(self.view_alliance_permission)
        cs = Invoice.objects.visible_to(self.user3)
        self.assertNotIn(self.ci1, cs)
        self.assertNotIn(self.ci2, cs)
        self.assertNotIn(self.ci3, cs)
        self.assertNotIn(self.ci4, cs)
        self.assertIn(self.ci5, cs)
        self.assertIn(self.ci6, cs)
        self.assertIn(self.ci7, cs)
        self.assertNotIn(self.ci8, cs)

    def test_get_alliance_with_corp(self):
        self.user3.user_permissions.add(self.view_alliance_permission)
        self.user3.user_permissions.add(self.view_corp_permission)
        cs = Invoice.objects.visible_to(self.user3)
        self.assertNotIn(self.ci1, cs)
        self.assertNotIn(self.ci2, cs)
        self.assertNotIn(self.ci3, cs)
        self.assertNotIn(self.ci4, cs)
        self.assertIn(self.ci5, cs)
        self.assertIn(self.ci6, cs)
        self.assertIn(self.ci7, cs)
        self.assertNotIn(self.ci8, cs)

    def test_global_perms_u1(self):
        self.user3.user_permissions.add(self.view_all_permission)
        cs = Invoice.objects.visible_to(self.user3)
        self.assertIn(self.ci1, cs)
        self.assertIn(self.ci2, cs)
        self.assertIn(self.ci3, cs)
        self.assertIn(self.ci4, cs)
        self.assertIn(self.ci5, cs)
        self.assertIn(self.ci6, cs)
        self.assertIn(self.ci7, cs)
        self.assertIn(self.ci8, cs)

    def test_global_perms_u2(self):
        self.user2.user_permissions.add(self.view_all_permission)
        cs = Invoice.objects.visible_to(self.user2)
        self.assertIn(self.ci1, cs)
        self.assertIn(self.ci2, cs)
        self.assertIn(self.ci3, cs)
        self.assertIn(self.ci4, cs)
        self.assertIn(self.ci5, cs)
        self.assertIn(self.ci6, cs)
        self.assertIn(self.ci7, cs)
        self.assertIn(self.ci8, cs)

    def test_global_perms_u3(self):
        self.user2.user_permissions.add(self.view_all_permission)
        cs = Invoice.objects.visible_to(self.user2)
        self.assertIn(self.ci1, cs)
        self.assertIn(self.ci2, cs)
        self.assertIn(self.ci3, cs)
        self.assertIn(self.ci4, cs)
        self.assertIn(self.ci5, cs)
        self.assertIn(self.ci6, cs)
        self.assertIn(self.ci7, cs)
        self.assertIn(self.ci8, cs)

    def test_no_su_perms(self):
        cs = Invoice.objects.visible_to(self.user5)
        self.assertNotIn(self.ci1, cs)
        self.assertNotIn(self.ci2, cs)
        self.assertNotIn(self.ci3, cs)
        self.assertNotIn(self.ci4, cs)
        self.assertNotIn(self.ci5, cs)
        self.assertNotIn(self.ci6, cs)
        self.assertNotIn(self.ci7, cs)
        self.assertNotIn(self.ci8, cs)

        self.user5.is_superuser=True
        
        cs = Invoice.objects.visible_to(self.user5)
        self.assertIn(self.ci1, cs)
        self.assertIn(self.ci2, cs)
        self.assertIn(self.ci3, cs)
        self.assertIn(self.ci4, cs)
        self.assertIn(self.ci5, cs)
        self.assertIn(self.ci6, cs)
        self.assertIn(self.ci7, cs)
        self.assertIn(self.ci8, cs)



from django.template.defaulttags import register
from django.utils.safestring import mark_safe


@register.filter()
def ore_color(ore):
    try:
        if 'Bitumens' in ore or 'Coesite' in ore or 'Sylvite' in ore or 'Zeolite' in ore:
            return mark_safe('style="background-color:#9B1C31"')
        elif 'Cobalite' in ore or 'Euxenite' in ore or 'Scheelite' in ore or 'Titanite' in ore:
            return mark_safe('style="background-color:#FFAA1D"')
        elif 'Chromite' in ore or 'Otavite' in ore or 'Sperrylite' in ore or 'Vanadinite' in ore:
            return mark_safe('style="background-color:#E86100"')
        elif 'Carnotite' in ore or 'Cinnabar' in ore or 'Pollucite' in ore or 'Zircon' in ore:
            return mark_safe('style="background-color:#4B8B3B"')
        elif 'Loparite' in ore or 'Monazite' in ore or 'Xenotime' in ore or 'Ytterbite' in ore:
            return mark_safe('style="background-color:#0D98BA"')
        else:
            return mark_safe('style="background-color:#B57EDC"')

    except:
        #logging.exception("Messsage")
        return ""




# Generated by Django 3.2.5 on 2021-08-30 11:12

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('moons', '0010_miningobservation_moon'),
    ]

    operations = [
        migrations.AddField(
            model_name='oretax',
            name='tax',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='tax_rate', to='moons.oretaxrates'),
            preserve_default=False,
        ),
    ]



# Generated by Django 3.2.5 on 2021-09-06 10:19

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('moons', '0015_auto_20210906_0915'),
    ]

    operations = [
        migrations.AddField(
            model_name='invoicerecord',
            name='base_ref',
            field=models.CharField(default='', max_length=72),
        ),
    ]



from django.core.management.base import BaseCommand, CommandError

from corptools.tasks import update_or_create_map, process_ores_from_esi, update_ore_comp_table
from django_celery_beat.models import CrontabSchedule, PeriodicTask, IntervalSchedule
from corptools.models import MapSystem
from moons.models import OreTaxRates
from moons.tasks import update_ore_prices
from celery import chain


class Command(BaseCommand):
    help = 'Update or Populate Moon Dependent Data'

    def add_arguments(self, parser):
        parser.add_argument('--inline', action='store_true',
                            help='Run update in Console not via Celery')

    def handle(self, *args, **options):
        self.stdout.write("Creating DB models!")
        if OreTaxRates.objects.all().count() == 0:
            OreTaxRates.objects.create(
                tag="Ore Tax",
                ore_rate=0.2,
                ubiquitous_rate=0.2,
                common_rate=0.2,
                uncommon_rate=0.2,
                rare_rate=0.2,
                exceptional_rate=0.2
            )
        self.stdout.write("Setting up Periodic Tasks!")
        schedule_bi_weekly, _ = IntervalSchedule.objects.get_or_create(every=14, 
                        period=IntervalSchedule.DAYS)
        schedule_20_min, _ = CrontabSchedule.objects.get_or_create(minute='10,30,50',
                                                                    hour='*',
                                                                    day_of_week='*',
                                                                    day_of_month='*',
                                                                    month_of_year='*',
                                                                    timezone='UTC'
                                                                    )
        
        schedule_start_of_month, _ = CrontabSchedule.objects.get_or_create(minute='0',
                                                                    hour='0',
                                                                    day_of_week='*',
                                                                    day_of_month='1',
                                                                    month_of_year='*',
                                                                    timezone='UTC'
                                                                    )
        
        task_obs, o_c= PeriodicTask.objects.update_or_create(
            task='moons.tasks.run_obs_for_all_corps',
            defaults={
                'crontab': schedule_20_min,
                'name': 'Moon Obs Updates',
                'enabled': True
            }
        )
        task_prices, o_p= PeriodicTask.objects.update_or_create(
            task='moons.tasks.update_ore_prices',
            defaults={
                'crontab': schedule_start_of_month,
                'name': 'Moon Ore Prices',
                'enabled': True
            }
        )
        task_invoice, o_i= PeriodicTask.objects.update_or_create(
            task='moons.tasks.generate_taxes',
            defaults={
                'interval': schedule_bi_weekly,
                'name': 'Send Moon Invoices',
                'enabled': False
            }
        )

        systems = MapSystem.objects.all().count()
        if options['inline']:
            self.stdout.write("Running Tasks inline this may take some time!")
            if systems <= 8285:
                self.stdout.write("Starting Map Update")
                update_or_create_map()
            else:
                self.stdout.write("No map update needed.")
            self.stdout.write("Starting Asteroid Data Update")
            process_ores_from_esi()
            self.stdout.write("Starting Ore Comp Update")
            update_ore_comp_table()
            self.stdout.write("Starting Ore Price Tasks")
            update_ore_prices()
            self.stdout.write("Done!")
        else:
            que=[]
            self.stdout.write("Sending Tasks to celery for processing!")
            if systems <= 8285:
                self.stdout.write("Sending Map Update Task")
                que.append(update_or_create_map.si())
            else:
                self.stdout.write("No map update needed.")
            self.stdout.write("Sending Asteroid Data Task")
            que.append(process_ores_from_esi.si())
            self.stdout.write("Sending Ore Comp Task")
            que.append(update_ore_comp_table.si())
            self.stdout.write("Sending Ore Price Tasks")
            que.append(update_ore_prices.si())
            self.stdout.write("Tasks Queued!")
            chain(que).apply_async(priority=4)




import os
import datetime

from django.db.models import Sum
from django.contrib import messages
from django.contrib.auth.decorators import login_required, permission_required
from django.shortcuts import render
from django.utils.translation import ugettext_lazy as _
from django.utils import timezone
from .models import MoonFrack, MiningObservation, InvoiceRecord
from corptools.models import EveLocation
from django.core.exceptions import PermissionDenied

@login_required
def extractions(request):
    if request.user.has_perm('moons.view_available'):
        days_to_hold = timezone.now() - datetime.timedelta(days=3)

        events = MoonFrack.objects.visible_to(request.user)
        future_fracks = events.filter(arrival_time__gt=timezone.now())
        current_fracks = events.filter(arrival_time__gte=days_to_hold, arrival_time__lt=timezone.now())
        
    else:
        raise PermissionDenied('You do not have permission to be here. This has been Logged!')

    context = {
        'events': future_fracks,
        'current_fracks': current_fracks,
    }

    return render(request, 'moons/list.html', context=context)


@login_required
@permission_required("admin")
def observers(request):
    all_obs=MiningObservation.objects.all().values('structure').distinct()
    locations = EveLocation.objects.filter(location_id__in=all_obs)
        
    context = {
        'observers': locations,
    }

    return render(request, 'moons/observers.html', context=context)


@login_required
@permission_required("admin")
def observer(request, observer_id):
    MiningObservation.objects.filter(observing_id=observer_id)

    context = {
        'observations': obs,
    }

    return render(request, 'moons/observers.html', context=context)






# Generated by Django 3.2.5 on 2021-08-27 10:51

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('moons', '0008_auto_20210827_1015'),
    ]

    operations = [
        migrations.AlterField(
            model_name='miningtax',
            name='flat_tax_rate',
            field=models.DecimalField(decimal_places=2, default=0.0, max_digits=5),
        ),
    ]



# Generated by Django 3.1.1 on 2020-10-20 08:07

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('corptools', '0037_mapsystemmoon_mapsystemplanet'),
    ]

    operations = [
        migrations.CreateModel(
            name='MoonFrack',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('moon_id', models.IntegerField()),
                ('start_time', models.DateTimeField()),
                ('arrival_time', models.DateTimeField()),
                ('auto_time', models.DateTimeField()),
                ('corporation', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='moon', to='corptools.corporationaudit')),
                ('moon_name', models.ForeignKey(default=None, null=True, on_delete=django.db.models.deletion.SET_NULL, to='corptools.mapsystemmoon')),
                ('notification', models.ForeignKey(default=None, null=True, on_delete=django.db.models.deletion.SET_NULL, to='corptools.notification')),
                ('structure', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='corptools.evelocation')),
            ],
            options={
                'permissions': (('view_available', 'Can View Configured Public Moons'), ('view_corp', 'Can View Own Corps Moons'), ('view_alliance', 'Can View Own Alliances Moons'), ('view_all', 'Can View All Moons')),
            },
        ),
        migrations.CreateModel(
            name='MiningObservation',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('observing_id', models.BigIntegerField(blank=True, default=None, null=True)),
                ('character_id', models.IntegerField()),
                ('last_updated', models.DateTimeField()),
                ('quantity', models.BigIntegerField()),
                ('recorded_corporation_id', models.IntegerField()),
                ('type_id', models.IntegerField()),
                ('char', models.ForeignKey(default=None, null=True, on_delete=django.db.models.deletion.SET_NULL, to='corptools.evename')),
                ('observer', models.ForeignKey(blank=True, default=None, null=True, on_delete=django.db.models.deletion.SET_NULL, to='corptools.evelocation')),
                ('type_name', models.ForeignKey(default=None, null=True, on_delete=django.db.models.deletion.SET_NULL, to='corptools.eveitemtype')),
            ],
        ),
        migrations.CreateModel(
            name='FrackOre',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('total_m3', models.DecimalField(decimal_places=2, max_digits=20)),
                ('frack', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='frack', to='moons.moonfrack')),
                ('ore', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='corptools.eveitemtype')),
            ],
        ),
        migrations.AddIndex(
            model_name='miningobservation',
            index=models.Index(fields=['observing_id'], name='moons_minin_observi_0bb532_idx'),
        ),
        migrations.AddIndex(
            model_name='miningobservation',
            index=models.Index(fields=['last_updated'], name='moons_minin_last_up_40e633_idx'),
        ),
        migrations.AddIndex(
            model_name='miningobservation',
            index=models.Index(fields=['recorded_corporation_id'], name='moons_minin_recorde_24c17c_idx'),
        ),
        migrations.AddIndex(
            model_name='miningobservation',
            index=models.Index(fields=['type_id'], name='moons_minin_type_id_a09027_idx'),
        ),
        migrations.AddIndex(
            model_name='miningobservation',
            index=models.Index(fields=['character_id'], name='moons_minin_charact_436005_idx'),
        ),
    ]



# Generated by Django 3.2.5 on 2021-08-27 10:15

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('corptools', '0058_auto_20210808_0635'),
        ('eveonline', '0014_auto_20210105_1413'),
        ('moons', '0007_miningtax_oreprice_oretax_oretaxrates'),
    ]

    operations = [
        migrations.AlterField(
            model_name='miningtax',
            name='constellation',
            field=models.ForeignKey(blank=True, default=None, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='tax_constellation', to='corptools.mapconstellation'),
        ),
        migrations.AlterField(
            model_name='miningtax',
            name='corp',
            field=models.ForeignKey(blank=True, default=None, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='moon_mining_tax', to='eveonline.evecorporationinfo'),
        ),
        migrations.AlterField(
            model_name='miningtax',
            name='moon',
            field=models.ForeignKey(blank=True, default=None, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='tax_moon', to='corptools.mapsystemmoon'),
        ),
        migrations.AlterField(
            model_name='miningtax',
            name='region',
            field=models.ForeignKey(blank=True, default=None, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='tax_region', to='corptools.mapregion'),
        ),
        migrations.AlterField(
            model_name='miningtax',
            name='system',
            field=models.ForeignKey(blank=True, default=None, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='tax_system', to='corptools.mapsystem'),
        ),
    ]



# Generated by Django 3.1.1 on 2020-12-05 06:06

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('moons', '0002_auto_20201128_1356'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='miningobservation',
            name='id',
        ),
        migrations.AddField(
            model_name='miningobservation',
            name='frack_pk',
            field=models.CharField(default=None, max_length=50, primary_key=True, serialize=False),
            preserve_default=False,
        ),
    ]



# Generated by Django 3.1.1 on 2020-12-05 06:26

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('moons', '0003_auto_20201205_0606'),
    ]

    operations = [
        migrations.RenameField(
            model_name='miningobservation',
            old_name='frack_pk',
            new_name='ob_pk',
        ),
    ]



# Generated by Django 3.2.5 on 2021-08-22 12:22

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('eveonline', '0014_auto_20210105_1413'),
        ('corptools', '0058_auto_20210808_0635'),
        ('moons', '0006_miningobservation_structure'),
    ]

    operations = [
        migrations.CreateModel(
            name='OreTaxRates',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('tag', models.CharField(default='Mining Tax', max_length=500)),
                ('refine_rate', models.DecimalField(decimal_places=2, default=87.5, max_digits=5)),
                ('ore_rate', models.DecimalField(decimal_places=2, max_digits=5)),
                ('ubiquitous_rate', models.DecimalField(decimal_places=2, max_digits=5)),
                ('common_rate', models.DecimalField(decimal_places=2, max_digits=5)),
                ('uncommon_rate', models.DecimalField(decimal_places=2, max_digits=5)),
                ('rare_rate', models.DecimalField(decimal_places=2, max_digits=5)),
                ('excptional_rate', models.DecimalField(decimal_places=2, max_digits=5)),
            ],
        ),
        migrations.CreateModel(
            name='OreTax',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('price', models.DecimalField(decimal_places=2, max_digits=20)),
                ('last_update', models.DateTimeField(auto_now=True)),
                ('item', models.ForeignKey(on_delete=django.db.models.deletion.DO_NOTHING, related_name='ore_tax', to='corptools.eveitemtype')),
            ],
        ),
        migrations.CreateModel(
            name='OrePrice',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('price', models.DecimalField(decimal_places=2, max_digits=20)),
                ('last_update', models.DateTimeField(auto_now=True)),
                ('item', models.ForeignKey(on_delete=django.db.models.deletion.DO_NOTHING, related_name='ore_price', to='corptools.eveitemtype')),
            ],
        ),
        migrations.CreateModel(
            name='MiningTax',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('use_variable_tax', models.BooleanField(default=False)),
                ('flat_tax_rate', models.DecimalField(blank=True, decimal_places=2, default=None, max_digits=5, null=True)),
                ('region', models.CharField(blank=True, default=None, max_length=50, null=True)),
                ('constellation', models.CharField(blank=True, default=None, max_length=50, null=True)),
                ('system', models.CharField(blank=True, default=None, max_length=50, null=True)),
                ('moon', models.CharField(blank=True, default=None, max_length=50, null=True)),
                ('rank', models.IntegerField(blank=True, default=0, null=True)),
                ('corp', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='moon_mining_tax', to='eveonline.evecorporationinfo')),
                ('tax_rate', models.ForeignKey(blank=True, default=None, null=True, on_delete=django.db.models.deletion.CASCADE, to='moons.oretaxrates')),
            ],
        ),
    ]



# Generated by Django 3.2.5 on 2021-09-04 10:37

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('moons', '0013_auto_20210904_0948'),
    ]

    operations = [
        migrations.AddField(
            model_name='invoicerecord',
            name='tax_dump',
            field=models.TextField(default=0),
            preserve_default=False,
        ),
    ]



from django.db import models
from django.core.exceptions import ObjectDoesNotExist
import logging

from allianceauth.eveonline.models import EveCharacter, EveCorporationInfo, EveAllianceInfo

logger = logging.getLogger(__name__)

class MoonQuerySet(models.QuerySet):
    def visible_to(self, user):
        # superusers get all visible
        if user.is_superuser:
            logger.debug('Returning all Moons for superuser %s.' % user)
            return self

        if user.has_perm('moons.view_all'):
            logger.debug('Returning all Moons for %s.' % user)
            return self

        try:
            char = user.profile.main_character
            assert char
            # build all accepted queries
            queries = []
            if user.has_perm('moons.view_alliance'):
                if char.alliance_id is not None:
                    queries.append(models.Q(corporation__corporation__alliance__alliance_id=char.alliance_id))
                else:
                    queries.append(models.Q(corporation__corporation__corporation_id=char.corporation_id))
            if user.has_perm('moons.view_corp'):
                if user.has_perm('moons.view_alliance'):
                    pass
                else:
                    queries.append(models.Q(corporation__corporation__corporation_id=char.corporation_id))
            logger.debug('%s queries for user %s characters.' % (len(queries), user))
            # filter based on queries
            query = queries.pop()
            for q in queries:
                query |= q
            return self.filter(query)
        except AssertionError:
            logger.debug('User %s has no main character. Nothing visible.' % user)
            return self.none()        


class MoonManager(models.Manager):

    def get_queryset(self):
        return MoonQuerySet(self.model, using=self._db)\
            .select_related('moon_name', 'corporation','corporation__corporation', 'structure', 'structure__system', 'structure__system__constellation')\
            .prefetch_related('frack', 'frack__ore')

    def visible_to(self, user):
        return self.get_queryset().visible_to(user)



moon_ping = ['MoonminingExtractionFinished']
moon_ping_admin = ['MoonminingAutomaticFracture', 'MoonminingExtractionStarted', 'MoonminingLaserFired']
"""
def process_ping(_id, _title, _content, _fields, _timestamp, _catagory, _col, _img, _url, _footer):
    custom_data = {'color': _col, 
                    'title': _title, 
                    'description': _content, 
                    'timestamp': _timestamp.replace(tzinfo=None).isoformat(),
                    'fields': _fields}
    if _img:
        custom_data['image'] = {'url': _img}
    if _footer:
        custom_data['footer'] = _footer

    return custom_data

if notification.notification_type == "MoonminingLaserFired":
    title = "Moon Laser Fired"
    notification_data = yaml.load(notification.notification_text)

    # firedBy: 824787891
    # firedByLink: <a href="showinfo:1380//824787891">PoseDamen</a>
    # moonID: 40291428
    # oreVolumeByType:
    #   45493: 1983681.4476127427
    #   46679: 2845769.539271295
    #   46681: 2046606.19987059
    #   46688: 2115548.2348155645
    # solarSystemID: 30004612
    # structureID: 1029754054149
    # structureLink: <a href="showinfo:35835//1029754054149">NY6-FH - ISF Two</a>
    # structureName: NY6-FH - ISF Two
    # structureTypeID: 35835

    system_name = MapSolarSystem.objects.get(
        solarSystemID=notification_data['solarSystemID']).solarSystemName
    system_name = '[%s](http://evemaps.dotlan.net/system/%s)' % \
                (system_name,
                system_name.replace(' ', '_'))
    structure_type = TypeName.objects.get(type_id=notification_data['structureTypeID']).name
    structure_name = notification_data['structureName']
    if len(structure_name)<1:
        structure_name = "Unknown"

    body = "Fired By [{0}](https://zkillboard.com/search/{1}/)".format(strip_tags(notification_data['firedByLink']), strip_tags(notification_data['firedByLink']).replace(" ", "%20"))

    timestamp = notification.timestamp
    corp_id = notification.character.character.corporation_id
    corp_ticker = notification.character.character.corporation_ticker
    footer = {"icon_url": "https://imageserver.eveonline.com/Corporation/%s_64.png" % (str(corp_id)),
                "text": "%s (%s)" % (notification.character.character.corporation_name, corp_ticker)}

    fields = [{'name': 'Structure', 'value': structure_name, 'inline': True},
                {'name': 'System', 'value': system_name, 'inline': True},
                {'name': 'Type', 'value': structure_type, 'inline': True},
                ]

    ping = process_ping(notification.notification_id,
                        title,
                        body,
                        fields,
                        timestamp,
                        "moon",
                        16756480,
                        False,
                        False,
                        footer)

    if ping:
        for hook in moon_hooks_admin:
            if hook.corporation is None:
                embed_lists[hook.discord_webhook]['embeds'].append(ping)
            elif hook.corporation.corporation_id == notification.character.character.corporation_id:
                embed_lists[hook.discord_webhook]['embeds'].append(ping)
            else:
                pass  # ignore

elif notification.notification_type == "MoonminingExtractionStarted":
    title = "Moon Extraction Started"
    notification_data = yaml.load(notification.notification_text)

    # autoTime: 132071260201940545
    # moonID: 40291428
    # oreVolumeByType:
    #   45493: 2742775.374017656
    #   46679: 3934758.0841854215
    #   46681: 2829779.495126257
    #   46688: 2925103.528079887
    # readyTime: 132071130601940545
    # solarSystemID: 30004612
    # startedBy: 824787891
    # startedByLink: <a href="showinfo:1380//824787891">PoseDamen</a>
    # structureID: 1029754054149
    # structureLink: <a href="showinfo:35835//1029754054149">NY6-FH - ISF Two</a>
    # structureName: NY6-FH - ISF Two
    # structureTypeID: 35835

    system_name = MapSolarSystem.objects.get(
        solarSystemID=notification_data['solarSystemID']).solarSystemName
    system_name = '[%s](http://evemaps.dotlan.net/system/%s)' % \
            (system_name,
            system_name.replace(' ', '_'))

    structure_type = TypeName.objects.get(type_id=notification_data['structureTypeID']).name
    structure_name = notification_data['structureName']
    if len(structure_name)<1:
        structure_name = "Unknown"

    body = "Started By [{0}](https://zkillboard.com/search/{1}/)".format(strip_tags(notification_data['startedByLink']), strip_tags(notification_data['startedByLink']).replace(" ", "%20"))
    timestamp = notification.timestamp
    corp_id = notification.character.character.corporation_id
    corp_ticker = notification.character.character.corporation_ticker
    footer = {"icon_url": "https://imageserver.eveonline.com/Corporation/%s_64.png" % (str(corp_id)),
                "text": "%s (%s)" % (notification.character.character.corporation_name, corp_ticker)}

    ready_time = filetime_to_dt(notification_data['readyTime'])

    fields = [{'name': 'Structure', 'value': structure_name, 'inline': True},
                {'name': 'System', 'value': system_name, 'inline': True},
                {'name': 'Type', 'value': structure_type, 'inline': True},
                {'name': 'Date Ready', 'value': ready_time.strftime("%Y-%m-%d %H:%M"), 'inline': False}
    ]

    ping = process_ping(notification.notification_id,
                        title,
                        body,
                        fields,
                        timestamp,
                        "moon",
                        6881024,
                        False,
                        False,
                        footer)

    if ping:
        for hook in moon_hooks_admin:
            if hook.corporation is None:
                embed_lists[hook.discord_webhook]['embeds'].append(ping)
            elif hook.corporation.corporation_id == notification.character.character.corporation_id:
                embed_lists[hook.discord_webhook]['embeds'].append(ping)
            else:
                pass  # ignore

elif notification.notification_type == "MoonminingAutomaticFracture":
    title = "Moon Auto-Fractured!"
    notification_data = yaml.load(notification.notification_text)

    # moonID: 40291417
    # oreVolumeByType:
    #   45492: 1524501.871099406
    #   46677: 2656351.8252801565
    #   46678: 1902385.1244004236
    #   46681: 2110988.956997792
    # solarSystemID: 30004612
    # structureID: 1030287515076
    # structureLink: <a href="showinfo:35835//1030287515076">NY6-FH - ISF-5</a>
    # structureName: NY6-FH - ISF-5
    # structureTypeID: 35835

    system_name = MapSolarSystem.objects.get(
        solarSystemID=notification_data['solarSystemID']).solarSystemName
    system_name = '[%s](http://evemaps.dotlan.net/system/%s)' % \
            (system_name,
            system_name.replace(' ', '_'))

    structure_type = TypeName.objects.get(type_id=notification_data['structureTypeID']).name
    structure_name = notification_data['structureName']
    if len(structure_name)<1:
        structure_name = "Unknown"

    body = "Ready To Mine!"
    timestamp = notification.timestamp
    corp_id = notification.character.character.corporation_id
    corp_ticker = notification.character.character.corporation_ticker
    footer = {"icon_url": "https://imageserver.eveonline.com/Corporation/%s_64.png" % (str(corp_id)),
                "text": "%s (%s)" % (notification.character.character.corporation_name, corp_ticker)}
    
    fields = [{'name': 'Structure', 'value': structure_name, 'inline': True},
                {'name': 'System', 'value': system_name, 'inline': True},
                {'name': 'Type', 'value': structure_type, 'inline': True},

    ]

    ping = process_ping(notification.notification_id,
                        title,
                        body,
                        fields,
                        timestamp,
                        "moon",
                        65533,
                        False,
                        False,
                        footer)

    if ping:
        for hook in moon_hooks_admin:
            if hook.corporation is None:
                embed_lists[hook.discord_webhook]['embeds'].append(ping)
            elif hook.corporation.corporation_id == notification.character.character.corporation_id:
                embed_lists[hook.discord_webhook]['embeds'].append(ping)
            else:
                pass  # ignore

elif notification.notification_type in moon_ping:
moon_hooks = discord_hooks.filter(moon_ping=True)

if notification.notification_type == "MoonminingExtractionFinished":
    title = "Moon Extraction Complete"
    notification_data = yaml.load(notification.notification_text)

    # autoTime: 132052212600000000
    # moonID: 40291390
    # oreVolumeByType:
    #   45490: 1588072.4935986102
    #   46677: 2029652.6969759
    #   46679: 3063178.818627033
    #   46682: 2839990.2933705184
    # solarSystemID: 30004612
    # structureID: 1029754067191
    # structureLink: <a href="showinfo:35835//1029754067191">NY6-FH - ISF Three</a>
    # structureName: NY6-FH - ISF Three
    # structureTypeID: 35835

    system_name = MapSolarSystem.objects.get(
        solarSystemID=notification_data['solarSystemID']).solarSystemName
    system_name = '[%s](http://evemaps.dotlan.net/system/%s)' % \
            (system_name,
            system_name.replace(' ', '_'))
    
    structure_type = TypeName.objects.get(type_id=notification_data['structureTypeID']).name
    structure_name = notification_data['structureName']
    if len(structure_name)<1:
        structure_name = "Unknown"

    body = "Ready to Fracture!"
    timestamp = notification.timestamp
    corp_id = notification.character.character.corporation_id
    corp_ticker = notification.character.character.corporation_ticker
    footer = {"icon_url": "https://imageserver.eveonline.com/Corporation/%s_64.png" % (str(corp_id)),
                "text": "%s (%s)" % (notification.character.character.corporation_name, corp_ticker)}

    auto_time = filetime_to_dt(notification_data['autoTime'])
    ores = {}       
    totalm3 = 0
    for t,q in notification_data['oreVolumeByType'].items():
        ores[t] = TypeName.objects.get(type_id=t).name
        totalm3 += q
    ore_string = []
    for t,q in notification_data['oreVolumeByType'].items():
        ore_string.append(
            "**{}**: {:2.1f}%".format(
                ores[t],
                q/totalm3*100
            )
        )
    fields = [{'name': 'Structure', 'value': structure_name, 'inline': True},
                {'name': 'System', 'value': system_name, 'inline': True},
                {'name': 'Type', 'value': structure_type, 'inline': True},
                {'name': 'Auto Fire', 'value': auto_time.strftime("%Y-%m-%d %H:%M"), 'inline': False},
                {'name': 'Ore', 'value': "\n".join(ore_string)},
    ]

    ping = process_ping(notification.notification_id,
                        title,
                        body,
                        fields,
                        timestamp,
                        "moon",
                        6881024,
                        False,
                        False,
                        footer)

    if ping:
        for hook in moon_hooks:
            if hook.corporation is None:
                embed_lists[hook.discord_webhook]['embeds'].append(ping)
            elif hook.corporation.corporation_id == notification.character.character.corporation_id:
                embed_lists[hook.discord_webhook]['embeds'].append(ping)
            else:
                pass  # ignore
"""


from django.core.management.base import BaseCommand, CommandError

from moons.models import InvoiceRecord
from moons.tasks import update_ore_prices


class Command(BaseCommand):
    help = 'Print Tax Stats'


    def handle(self, *args, **options):
        self.stdout.write("Calculating!")
        last_date = InvoiceRecord.get_last_invoice_date()
        self.stdout.write(f"Last Invoice {last_date}!")
        self.stdout.write("Doing some math... Please wait...\n\n")
        accounts_seen = 0
        locations = set()
        data = InvoiceRecord.generate_invoice_data()
        total_mined = 0
        total_taxed = 0
        # run known people
        for u, d in data['knowns'].items():
            try:
                total_mined += d['total_value']
                total_taxed += d['tax_value']

                accounts_seen += 1
                for l in d['locations']:
                    locations.add(l)
            except KeyError:
                pass # probably wanna ping admin about it.
        
        for u, d in data['unknowns'].items():
            try:
                total_mined += d['totals_isk']
                total_taxed += d['tax_isk']
                for l in d['seen_at']:
                    locations.add(l)
            except KeyError:
                pass # probably wanna ping admin about it.

        self.stdout.write(f"We've seen {accounts_seen} known members!")
        self.stdout.write(f"We've seen { len(data['unknowns']) } unknown characters!\n\n")
        self.stdout.write(f"Who have mined ${total_mined:,} worth of ore!")
        self.stdout.write(f"Current Tax puts this at ${total_taxed:,} in taxes!\n\n")
        structures = "\n ".join(list(locations))
        self.stdout.write(f"the structures included are:\n { structures }")

        





from django.contrib import admin

# Register your models here.
from .models import MoonFrack, MiningTax, OreTaxRates, InvoiceRecord


class MoonAdmin(admin.ModelAdmin):
    list_select_related = True
    list_display = ['corporation','moon_name', 'arrival_time', 'notification']
    search_fields = ('corporation', 'moon_name')
    raw_id_fields = ('corporation', 'moon_name', 'structure', 'notification')
    

admin.site.register(MoonFrack, MoonAdmin)


class TaxAdmin(admin.ModelAdmin):
    list_display = ('rank', 'corp', 'use_variable_tax', 'tax_rate', '__str__')
    search_fields = ['region', 'constellation', 'system', 'moon', 'corp']
    ordering = ('-rank',)
    raw_id_fields = ('region', 'constellation', 'system', 'moon')


admin.site.register(MiningTax, TaxAdmin)


class OreTaxRatesAdmin(admin.ModelAdmin):
    list_display=('tag', 'refine_rate', 'exceptional_rate', 'rare_rate', 'uncommon_rate', 'common_rate', 'ubiquitous_rate', 'ore_rate')

admin.site.register(OreTaxRates, OreTaxRatesAdmin)

class InvoiceAdmin(admin.ModelAdmin):
    #generate a custom formater cause i am lazy...
    def __init__(self, *args, **kwargs):
        def generate_formatter(name, str_format):
            formatter = lambda o: str_format.format(getattr(o, name) or 0)
            formatter.short_description = name
            formatter.admin_order_field = name
            return formatter

        all_fields = []
        for f in self.list_display:
            if isinstance(f, str):
                all_fields.append(f)
            else:
                new_field_name = "_" + f[0]
                setattr(self, new_field_name, generate_formatter(f[0], f[1]))
                all_fields.append(new_field_name)
        self.list_display = all_fields

        super().__init__(*args, **kwargs)

    list_display = ['base_ref', 'start_date', 'end_date', ('total_mined', "{:,}"), ('total_taxed', "{:,}")]
    

admin.site.register(InvoiceRecord, InvoiceAdmin)



# Generated by Django 3.2.5 on 2021-09-04 09:48

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('corptools', '0059_invtypematerials_met_type'),
        ('moons', '0012_rename_excptional_rate_oretaxrates_exceptional_rate'),
    ]

    operations = [
        migrations.CreateModel(
            name='InvoiceRecord',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('start_date', models.DateTimeField()),
                ('end_date', models.DateTimeField()),
                ('ore_prices', models.TextField()),
            ],
        ),
        migrations.AddField(
            model_name='miningobservation',
            name='observing_corporation',
            field=models.ForeignKey(default=None, null=True, on_delete=django.db.models.deletion.SET_NULL, to='corptools.corporationaudit'),
        ),
    ]



# Generated by Django 3.1.1 on 2020-11-28 13:56

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('moons', '0001_initial'),
    ]

    operations = [
        migrations.RenameField(
            model_name='miningobservation',
            old_name='char',
            new_name='character_name',
        ),
        migrations.RemoveField(
            model_name='miningobservation',
            name='observer',
        ),
        migrations.AddField(
            model_name='miningobservation',
            name='frack',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='moons.moonfrack'),
            preserve_default=False,
        ),
    ]
